/**
 * gcc -o preprocessing preprocessing.cpp -I/usr/include/ -llept
 */


#include "leptonica/allheaders.h"

int main(int argc, char *argv[])
{
	Pix *inputImage;
	PIX *pixs, *pixc, *pixr, *pixg, *pixb, *pixsg, *pixsm, *pixd;
	l_int32 	xres, yres, depth;
	
    inputImage = pixRead(argv[1]);
    if (!inputImage) {
		fprintf(stderr, "Could create PIX from '%s'.\n", argv[1]);
		exit(1);
	} 

    /**
	 * Set resolution
	 */
	pixGetResolution(inputImage, &xres, &yres);
	// if leptonica is not able to get resolution - set it to 180 - this
	// information is get from imagick's identify, confirmed by XnVview
	// and IrfanView.
	if (xres < 70 xres = 180;
	if (yres < 70) yres = 180;
	pixSetResolution(inputImage, xres, yres);

    /* Print some info about image */
	depth = pixGetDepth(inputImage);
	printf("Image size: %ix%i.\n", pixGetWidth(inputImage),
		pixGetHeight(inputImage));
	printf("wpl: %i, bpl: %i.\n", pixGetWpl(inputImage),
		pixGetWpl(inputImage)*4);
	printf("Resolution: %ix%i.\n", xres, yres);
	printf("Depth: %ix%i.\n", depth);

	/* Convert the RGB image to grayscale. */
	printf("Converting to grayscale...\n");
	pixsg = pixConvertRGBToLuminance(inputImage);
	//pixWrite("step_1.png", pixsg, IFF_PNG);
	
	/* Remove the text in the fg. */
	printf("Removing the text in the fg...\n");
	pixc = pixCloseGray(pixsg, 25, 25);
	//pixWrite("step_2.png", pixc, IFF_PNG);

	/* Smooth the bg with a convolution. */
	printf("Smoothing the bg with a convolution...\n");
	pixsm = pixBlockconv(pixc, 15, 15);
	//pixWrite("step_3.png", pixsm, IFF_PNG);
	pixDestroy(&pixsm);

	/* Normalize for uneven illumination on gray image. */
	printf("Normalize for uneven illumination on gray image...\n");
	pixBackgroundNormGrayArrayMorph(pixsg, NULL, 4, 5, 200, &pixg);
	pixc = pixApplyInvBackgroundGrayMap(pixsg, pixg, 4, 4);
	//pixWrite("step_4.png", pixc, IFF_PNG);
    pixDestroy(&pixg);

	/* Increase the dynamic range. */
	printf("Appling pixGammaTRC...\n");
	// make dark gray *black* and light gray *white*
	pixd = pixGammaTRC(NULL, pixc, 1.0, 50, 220);
	//pixWrite("step_5.png", pixd, IFF_PNG);

	/* Threshold to 1 bpp. */
	printf("Appling pixThresholdToBinary...\n");
	pixb = pixThresholdToBinary(pixd, 120);

	printf("Saving output to 'preprocessed.png'.\n");
	pixWrite("preprocessed.png", pixb, IFF_PNG);
    return 0;
}
